﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.31
@EndOfDesignText@
#Event: DragDetected(EventData As MouseEvent) as boolean
#Event: MouseDragEntered(EventData as MouseEvent)
#Event: MouseDragExited(EventData as MouseEvent)
#Event: MouseDragOver(EventData as MouseEvent)
#Event: MouseDragReleased(EventData as MouseEvent)

Sub Class_Globals
	Private fx As JFX
	Private mCallBack As Object
	Private DragDetectedEventNames As Map
	Private MouseDragEventsNames As Map
	Private mSenderNode As Node
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(callBack As Object)
	mCallBack = callBack
	DragDetectedEventNames.Initialize
	MouseDragEventsNames.Initialize
End Sub

'This sub registers the DragDetected(EventData As MouseEvent) event to a node.
'Return true from the DragDetected event if you want the drag to start
'Use the SenderNode property of the FullDragRelease object to get the sending node
'The FullDragRelease object should be declared in Process_Globals
'
'When you drag a node, it's still under cursor, so it is considered being a potential gesture target during the whole gesture. If you need to drag a node to a different node and let the other node know about it, you need to ensure that the nodes under the dragged node are picked as the potential gesture targets. You can achieve this by calling setMouseTransparent(pNode As Node, value As Boolean) on the dragged node in a MOUSE_PRESSED handler and returning it back to false in a MOUSE_RELEASED handler. This way the nodes under the dragged node will receive the MouseDragEvents, while all the MouseEvents will still be delivered to the (currently mouse transparent) gesture source.
Public Sub MakeDragReleaseSource(pNode As Node, EventName As String)
	DragDetectedEventNames.Put(pNode,EventName)
	If SubExists(mCallBack,EventName & "_DragDetected") Then
		Dim Event As Object = AsJavaObject(pNode).CreateEvent("javafx.event.EventHandler","DragDetected",Null)
		AsJavaObject(pNode).RunMethod("setOnDragDetected",Array(Event))
	End If
End Sub

'This sub registers the following events to a node
'   .._MouseDragEntered(EventData as MouseEvent)
'   .._MouseDragExited(EventData as MouseEvent)
'   .._MouseDragOver(EventData as MouseEvent)
'   .._MouseDragReleased(EventData as MouseEvent)
'Use the SenderNode property of the FullDragRelease object to get the sending node
Public Sub MakeDragReleaseTarget(pNode As Node, EventName As String)
	MouseDragEventsNames.Put(pNode,EventName)
	If SubExists(mCallBack,EventName & "_MouseDragEntered") Then
		Dim Event As Object = AsJavaObject(pNode).CreateEvent("javafx.event.EventHandler","MouseDragEntered",Null)
		AsJavaObject(pNode).RunMethod("setOnMouseDragEntered",Array(Event))
	End If
	If SubExists(mCallBack,EventName & "_MouseDragExited") Then
		Dim Event As Object = AsJavaObject(pNode).CreateEvent("javafx.event.EventHandler","MouseDragExited",Null)
		AsJavaObject(pNode).RunMethod("setOnMouseDragExited",Array(Event))
	End If
	If SubExists(mCallBack,EventName & "_MouseDragOver") Then
		Dim Event As Object = AsJavaObject(pNode).CreateEvent("javafx.event.EventHandler","MouseDragOver",Null)
		AsJavaObject(pNode).RunMethod("setOnMouseDragOver",Array(Event))
	End If
	If SubExists(mCallBack,EventName & "_MouseDragReleased") Then
		Dim Event As Object = AsJavaObject(pNode).CreateEvent("javafx.event.EventHandler","MouseDragReleased",Null)
		AsJavaObject(pNode).RunMethod("setOnMouseDragReleased",Array(Event))
	End If
End Sub

'When you drag a node, it's still under cursor, so it is considered being a potential gesture target during the whole gesture. If you need to drag a node to a different node and let the other node know about it, you need to ensure that the nodes under the dragged node are picked as the potential gesture targets. You can achieve this by calling setMouseTransparent(pNode As Node, value As Boolean) on the dragged node in a MOUSE_PRESSED handler and returning it back to false in a MOUSE_RELEASED handler. This way the nodes under the dragged node will receive the MouseDragEvents, while all the MouseEvents will still be delivered to the (currently mouse transparent) gesture source.
Public Sub setMouseTransparent(pNode As Node, value As Boolean)
	AsJavaObject(pNode).RunMethod("setMouseTransparent",Array(value))
End Sub

public Sub getSenderNode As Node
	Return mSenderNode
End Sub

Private Sub DragDetected_Event(MethodName As String,Args() As Object) 'As Object
	mSenderNode = Sender
	If CallSub2(mCallBack,DragDetectedEventNames.Get(mSenderNode) & "_DragDetected",AsMouseEvent(Args(0))) Then
		AsJavaObject(mSenderNode).RunMethod("startFullDrag",Array())
	End If
End Sub

Private Sub MouseDragEntered_Event(MethodName As String,Args() As Object) 'As Object
	mSenderNode = Sender
	CallSub2(mCallBack,MouseDragEventsNames.Get(mSenderNode) & "_MouseDragEntered",AsMouseEvent(Args(0)))
End Sub
Private Sub MouseDragExited_Event(MethodName As String,Args() As Object) 'As Object
	mSenderNode = Sender
	CallSub2(mCallBack,MouseDragEventsNames.Get(mSenderNode) & "_MouseDragExited",AsMouseEvent(Args(0)))
End Sub
Private Sub MouseDragOver_Event(MethodName As String,Args() As Object) 'As Object
	mSenderNode = Sender
	CallSub2(mCallBack,MouseDragEventsNames.Get(mSenderNode) & "_MouseDragOver",AsMouseEvent(Args(0)))
End Sub
Private Sub MouseDragReleased_Event(MethodName As String,Args() As Object) 'As Object
	mSenderNode = Sender
	CallSub2(mCallBack,MouseDragEventsNames.Get(mSenderNode) & "_MouseDragReleased",AsMouseEvent(Args(0)))
End Sub

Private Sub AsJavaObject(JO As JavaObject) As JavaObject
	Return JO
End Sub
Private Sub AsMouseEvent(M As MouseEvent) As MouseEvent
	Return M
End Sub


